library fyc_index_bar;

import 'package:flutter/material.dart';

double ScreenWith(BuildContext context) => MediaQuery.of(context).size.width;
double ScreenHeight(BuildContext context) => MediaQuery.of(context).size.height;

class IndexBar extends StatefulWidget {
  final void Function(String str) indexBarCallBack;
  IndexBar({required this.indexBarCallBack});

  @override
  _IndexBarState createState() => _IndexBarState();
}

int _getIndex(BuildContext context, Offset globalPostion) {
  // 拿到box
  // RenderObject? box = context.findRenderObject();
  RenderBox? box = context.findRenderObject() as RenderBox?;
  // 拿到y值
  double y = box!.globalToLocal(globalPostion).dy;
  // 算出字符高度
  var itemHeight = ScreenHeight(context) / 2 / INDEX_WORDS.length;
  int index = (y ~/ itemHeight).clamp(0, INDEX_WORDS.length - 1);
  return index;
}

class _IndexBarState extends State<IndexBar> {
  Color _bkColor = Color.fromRGBO(1, 1, 1, 0);
  Color _textColor = Colors.black;

  double _indicatorY = 0.0;
  String _indicatorText = 'A';
  bool _indecatorHidden = true;

  @override
  Widget build(BuildContext context) {
    List<Widget> words = [];
    for (int i = 0; i < INDEX_WORDS.length; i++){
      words.add(Expanded(child: Text(INDEX_WORDS[i],style: TextStyle(fontSize: 10, color: _textColor),)));
    }

    return Positioned(
      right: 0.0,
      top: ScreenHeight(context) / 8,
      height: ScreenHeight(context) / 2,
      width: 120,
      child: Row(
        children: [
          Container( // 气泡
            alignment: Alignment(0,_indicatorY),
            width: 100,
            child: _indecatorHidden ? null : Stack(
              alignment: Alignment(-0.2,0),
              children: [
                Image(image: AssetImage('images/气泡.png'),width: 60,),
                Text(_indicatorText,style: TextStyle(fontSize: 35,color: Colors.white),)
              ],
            ),
          ),
          GestureDetector(
            child: Container(
              width: 20,
              color: _bkColor,
              child: Column(
                children: words,
              ),
            ),
            onVerticalDragUpdate: (DragUpdateDetails details){
              int index = _getIndex(context, details.globalPosition);
              widget.indexBarCallBack(INDEX_WORDS[index]);
              _indicatorText = INDEX_WORDS[index];
              _indicatorY = 2.2 / 28 * index - 1.1;
              _indecatorHidden = false;
              setState(() {});
            },
            onVerticalDragDown: (DragDownDetails details){
              int index = _getIndex(context,details.globalPosition);
              widget.indexBarCallBack(INDEX_WORDS[index]);
              // 设置回调
              setState(() {
                _indicatorText = INDEX_WORDS[index];
                _indicatorY = 2.2/28*index - 1.1;
                _indecatorHidden = false;
                _bkColor = Color.fromRGBO(1, 1, 1, 0.5);
                _textColor = Colors.white;
              });
            },
            onVerticalDragEnd: (DragEndDetails details){
              setState(() {
                _indecatorHidden = true;
                _bkColor = Color.fromRGBO(1, 1, 1, 0.0);
                _textColor = Colors.black;
              });
            },
          )
        ],
      ),
    );
  }
}


const INDEX_WORDS = [
  '🔍',
  '☆',
  'A',
  'B',
  'C',
  'D',
  'E',
  'F',
  'G',
  'H',
  'I',
  'J',
  'K',
  'L',
  'M',
  'N',
  'O',
  'P',
  'Q',
  'R',
  'S',
  'T',
  'U',
  'V',
  'W',
  'X',
  'Y',
  'Z'
];
